from numpy.lib import imag
from nvidia_kernels import gravity, pixel_shader, vector
from nvidia_kernels.kernel import auto_build
from PIL import Image
from utils import Timer

import numpy as np

from numba import cuda

timer = Timer()
DTYPE = np.float32

class Universum:

    @classmethod
    def from_np_arr(cls, positions, velocities, masses, image_buffer):
        return cls(
            cuda.to_device(positions.astype(DTYPE)),
            cuda.to_device(velocities.astype(DTYPE)),
            cuda.to_device(masses.astype(DTYPE)),
            cuda.to_device(image_buffer.astype(DTYPE)),
        )

    def __init__(self, positions, velocities, masses, image_buffer) -> None:
        self._positions = positions
        self._velocities = velocities
        self._masses = masses
        self._image_buffer = image_buffer

        self._update_accelerations = auto_build(gravity.calculate_accelerations_with_mass, positions.shape[0], 320)
        self._add_accelerations = auto_build(vector.add_vector_arrays, positions.shape[0], 640)
        self._draw_positions = auto_build(pixel_shader.convert_planets_to_image, image_buffer.size, 64)

        size = np.array(image_buffer.shape[:2], DTYPE)
        start = np.array(-size, DTYPE)
        self._window_start = start
        end = np.array([0, 0], DTYPE)
        self._window_end = end

    def simulate(self, dt):
        self._update_accelerations(self._positions, self._masses, self._velocities)
        self._add_accelerations(self._positions, self._velocities, dt)


    @property
    def positions(self):
        return self._positions.copy_to_host()

    @property
    def velocities(self):
        return self._velocities.copy_to_host()

    @property
    def masses(self):
        return self._masses.copy_to_host()
        
    @property
    def image_buffer(self):
        self._draw_positions(self._positions, self._masses, self._window_start, self._window_end, self._image_buffer)
        return (self._image_buffer.copy_to_host() * 255).astype(np.uint8)
    
    def screenshot(self, fp) -> Image:
        Image.fromarray(self.image_buffer).save(fp)
        
    def set_window(self, start, end):
        self._window_start = np.array(start)
        self._window_end = np.array(end)
        